/***************************************************************************
                          animatedsprite.cpp  -  description
                             -------------------
    begin                : Mit Aug 21 2002
    copyright            : (C) 2002, 2004 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "animatedsprite.h"
#include "soundwrapper.h"

/** Constructor. Note, that you will have to call setAnimation before calling tick () the first time, or the app will crash */
AnimatedSprite::AnimatedSprite():QCanvasSprite (0, TPBase::canvas) {
	_direction = AnimForward;
	_orientation = AnimLeft;
	frame = 0;
	repetition = 1;
	sound_processing = false;
}

/** Destructor. Does not delete the attached Animation. You'll have to do that by hand. */
AnimatedSprite::~AnimatedSprite(){
}

/** Advances the animation (taking care of all the details, including sound). Returns true if this tick finishes the animation */
bool AnimatedSprite::tick () {
	if (_direction == AnimForward) {
		if (++frame >= (anim->frames -1)) {
			if (anim->oscillating) {
				setDirection (AnimReverse);
				setFrame (frame = (anim->frames -1));
				return false;
			} else if (anim->continuous) {
				setFrame (anim->frames -1);
				frame = -1;				// next tick will set this to zero, before using setFrame!
				return false;
			} else if (anim->repetitions > repetition) {
				if (anim->sound.properties & Repetitious) {
					sw->start_sound (anim->sound.id[_direction], sound_pitch);
				}
				setFrame (anim->frames -1);
				frame = -1;
				repetition++;
				return false;
			} else {
				setFrame (anim->frames -1);
				frame = -1;
				repetition = 1;
				return true;
			}
		} else {
			setFrame (frame);
			return false;
		}
	} else {			// direction is AnimReverse
		if (--frame <= 0) {
			if (anim->oscillating) {
				setDirection (AnimForward);
				setFrame (0);
				if (anim->repetitions > repetition) {
					frame = -1;
					repetition++;
					return false;
      				} else if (anim->continuous) {
					return false;
				} else {
					return true;
				}
			} else if (anim->continuous) {
				setFrame (0);
				frame = anim->frames;
				return false;
			} else if (anim->repetitions < repetition) {
				if (anim->sound.properties & Repetitious) {
					sw->start_sound (anim->sound.id[_direction], sound_pitch);
				}
				setFrame (0);
				frame = anim->frames;
				repetition++;
				return false;
			} else {
				setFrame (0);
				frame = anim->frames;
				repetition = 1;
				return true;
			}
		} else {
			setFrame (frame);
			return false;
		}
	}
}

/** Set the direction to direction (i.e. AnimForward or AnimReverse) */
void AnimatedSprite::setDirection (AnimDirections direction) {
	if (direction != _direction) {
		if (sound_processing) {
			sw->stop_sound(anim->sound.id[_direction]);
		}
		_direction = direction;
		if (sound_processing) {
			if (anim->sound.properties & (Repetitious | Continuous)) {
				sw->start_sound(anim->sound.id[_direction], sound_pitch);
			}
		}
	}
}

/** Sets the Animation to use and resets settings for frame, repetition... also stops any sound that were still playing */
void AnimatedSprite::setAnimation (Animation *animation, AnimDirections direction) {
	stopSound ();
	setDirection (direction);
	anim = animation;
	bool was_visible = visible ();
	setVisible (false);
	setSequence (anim->seq[_orientation]);
	if (direction == AnimForward) {
		setFrame (frame = 0);
	} else {
		setFrame (frame = (anim->frames - 1));
	}
	setVisible (was_visible);
	repetition = 1;
}

/** Sets the orientation (i.e. going left or right etc. See TPBase::AnimationOrientations) */
void AnimatedSprite::setOrientation (AnimOrientations orientation) {
	if (_orientation != orientation) {			// setSequence is somewhat expensive, so avoid if not needed
		_orientation = orientation;
		setSequence (anim->seq[_orientation]);
	}
}

/** Starts sound-processing. Since the sprite is never explicitely told, when it gets started to be used, this has to be called
	manually (but only once for the animation, even if it has several repetitions) */
void AnimatedSprite::startSound (float pitch) {
	if (anim->sound.properties & Pitchable) {
		sound_pitch = pitch;
	} else {
		sound_pitch = 1;
	}
	if (!sound_processing) {
		sound_processing = true;
		sw->start_sound (anim->sound.id[_direction], sound_pitch);
	}
}

/** Stops sound-processing. See above. Will also be called from setAnimation, so usually no need to call manually */
void AnimatedSprite::stopSound () {
	if (sound_processing) {
		sw->stop_sound(anim->sound.id[_direction]);
		sound_processing = false;
	}
}
